---
layout: post
title:  "backtrace-symbold 2.6.??"
date:   2023-08-23  12:00:00 -0000
categories: symbold
sidebar_link: true
---

This release is recommended for all users.

# Features

* The symbold service is now packaged as a single executable and no longer
  depends on a specific system Node.js version.

* Adds `debugIdPrefix` to the set of query builder substitutions. This can be
  used to substitute the first two characters of a debug identifier into a
  symbol query. For example, "{debugIdPrefix}/{debugId}.nxsym".

* Adds `switchDebugId` and `switchDebugIdPrefix` to the set of query build
  substitutions. These were added to support the layout of a Switch symbol
  server.

* Adds a configuration option, `.binary.symbol_server_inter_task_delay`, which
  can used to adjust the time, in milliseconds, to wait to download from a
  symbol server that is busy downloading a symbol. The default for this
  configurable is 1000 ms and does not typically need to be changed.

* Adds a configuration option, `.binary.is_404_not_found_retryable`, which can
  be set to false to disable retries when downloading from a symbol server
  that returns a 404 (Not Found) response code. By default, this is "true".

* Adds a configuration option, `.binary.is_403_forbidden_retryable`, which can
  be set to false to disable retries when downloading from a symbol server
  that returns a 403 (Forbidden) response code. By default, this is "true".

* Implemented batch reprocessing of missing symbol events. This opt-in feature
  will queue completed missing symbol events for a project and send a batched
  reprocessing notification to coronerd. This can be enabled by setting the
  configuration item `.processing.batch_processing` to true.

* Adds optional `.path.btSymbolsConvert` which can be set to the path of the
  bt_symbols_convert tool available in backtrace-coronerd-tools 1.60 and later.
  This tool unifies the various symbol conversion tools and has been updated to
  support the conversion of Mach-O binaries downloaded from symbol servers. Set
  this to the full filename of bt_symbols_convert, e.g.
  "/opt/backtrace/bin/bt_symbols_convert", to enable Mach-O support in symbold.

* Adds `.log.conversionDir` which, when set, will be used as the directory for
  storing gzip compressed output (stderr) from symbol conversions and can be
  used to help diagnose errors. The amount of time the logs are retained can be
  configured by `.log.conversionRetention`, with units in seconds. If not
  specified, log files are pruned after 1 week.

* Adds `.processing.max_event_processing_time` to restrict the maximum duration,
  measured in milliseconds, that a 'missing symbols' event can consume to
  download the associated set of symbols. If not specified, this defaults to 10
  minutes (600000).

# Improvements

* Now set default settings for lowerFile and lowerDebug in a symbol server's
  credentials when not specified (via morgue or curl).

* Eliminated symbold's dependency on coronerd.conf. The symbold config item
  `path.coronerd[]` was repurposed to point to the path of coronerd rather than
  the location of the coronerd.conf file. The change is backwards compatible,
  however, as paths in the array containing coronerd.conf, such as
  `/etc/coronerd` are ignored.

* Now gracefully handle a symbol server disconnecting in the middle of a
  download.

* Handle missing symbol event without an 'os' property. This fixes a problem
  with the /api/symbold/queue/add endpoint as well as processing missing symbol
  events generated from minidumps without system information.

* Add an endpoint that can be used to dynamically change the logger level. The
  endpoint is /api/symbold/config/log-level/{info, debug}.

* Symbols are now converted once per missing symbol event rather than for each
  group of symbols.

* Implements a symbol server connection timeout. If symbold fails to initiate a
  connection to a symbol server then the download is aborted. This is used to
  eliminate processing delays for offline symbol servers. By default, the
  connection timeout is set to 500 ms. This can be configured by
  `.binary.connect_timeout`.

# Internal

* Renamed the global symbol server, "iOS Symbol Server", to "Backtrace Symbol
  Server" and the corresponding whitelist from "iOS_whitelist.txt" to
  "bt_common_whitelist.txt". Added XGDK symbols to bt_common_whitelist.txt which
  were uploaded to the bt-common-symbols-test S3 bucket.

* Removed stadia and electron symbol servers from default configuration as these
  symbol servers are no longer available.

* Replace the in-memory cache of symbols download, which was cleared every hour
  with an in-memory LRU download history cache. This cache is used to record the
  time of download, whether the download was successful or not, and whether the
  conversion for that downloaded symbol succeeded. Adds a new config item,
  ".downloadHistoryCacheExpiry" which can be used to configure the time in which
  download history entries are evicted from the cache. By default, this is three
  hours. Adds an endpoint /api/symbold/symbols/status/download-history to
  obtain the current download history state.

* Removed code path that decompresses user submitted .symb.zst files. This
  feature hadn't been enabled on any host for multiple years and, in any case,
  its functionality was replaced by the new symbol caches.

* Adds a mechanism to automatically clear the skiplist at service start. The
  service looks for the symlink at
  `${dirname(symbol.db}`/state/skiplists-cleared` and ensures it points to the
  expected version number, presently "1".

* Aside from the above internal changes, the majority of the work in symbold
  since the last on-prem release was geared toward supporting local variables
  and is documented in the docs/ rather than rehashed here. This includes the
  symbol caching, project directory structure, symbol statistics and ranking,
  endpoints for intercepting communications from symbol-processor, retention of
  original symbols, storage and retrieval of symbols in S3 buckets, and so
  forth. That said, this release is designed to be backwards compatible:
  customers should not have to modify their symbold.conf from a previous
  release.
